HOME_URL = "https://i.sjtu.edu.cn/xtgl/index_initMenu.html"
SCHEDULE_URL = "https://i.sjtu.edu.cn/kbcx/xskbcx_cxXsKb.html?gnmkdm=N2151"
ELECT_URL = "https://i.sjtu.edu.cn/xsxk/zzxkyzb_cxZzxkYzbIndex.html?gnmkdm=N253512&layout=default&su="
LOGIN_URL = "https://i.sjtu.edu.cn/jaccountlogin"
LOGIN_POST_URL = "https://jaccount.sjtu.edu.cn/jaccount/ulogin"
LOGOUT_URL = "https://i.sjtu.edu.cn/logout"
CAPTCHA_URL = "https://jaccount.sjtu.edu.cn/jaccount/captcha"
SCORE_URL = "https://i.sjtu.edu.cn/cjcx/cjcx_cxDgXscj.html?doType=query&gnmkdm=N305005"
SCORE_DETAIL_URL = "https://i.sjtu.edu.cn/cjcx/cjcx_cxXsXmcjList.html?gnmkdm=N305007&su="
EXAM_URL = "https://i.sjtu.edu.cn/kwgl/kscx_cxXsksxxIndex.html?doType=query&gnmkdm=N358105&su="
CALENDAR_URL = "https://i.sjtu.edu.cn/xtgl/index_cxshjdAreaFive.html?localeKey=zh_CN&gnmkdm=index&su="
COURSELIB_URL = "https://i.sjtu.edu.cn/design/funcData_cxFuncDataList.html?" \
                "func_widget_guid=DA1B5BB30E1F4CB99D1F6F526537777B&gnmkdm=N219904&su="
GPA_PARAMS_URL = "https://i.sjtu.edu.cn/cjgl/common_cxGnzdxxList.html?gnmkdm=N309130"
GPA_CALC_URL = "https://i.sjtu.edu.cn/cjpmtj/gpapmtj_tjGpapmtj.html?gnmkdm=N309131&su="
GPA_QUERY_URL = "https://i.sjtu.edu.cn/cjpmtj/gpapmtj_cxGpaxjfcxIndex.html?doType=query&gnmkdm=N309131&su="
HEADERS = {"User-Agent": "Mozilla/5.0 (X11; Linux x86_64; rv:71.0) Gecko/20100101 Firefox/71.0",
           "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8"}
TERMS = [3, 12, 16]
